@@ -1,13 +1,13 @@
-import YAML from '../../index.js'
-import { Merge, YAMLMap } from '../../types.js'
+import * as YAML from 'yaml'
 
 test('basic', () => {
   const src = `- &a 1\n- *a\n`
   const doc = YAML.parseDocument(src)
   expect(doc.errors).toHaveLength(0)
-  const { items } = doc.contents
-  expect(items).toMatchObject([{ value: 1 }, { source: { value: 1 } }])
-  expect(items[1].source).toBe(items[0])
+  expect(doc.contents.items).toMatchObject([
+    { anchor: 'a', value: 1 },
+    { source: 'a' }
+  ])
   expect(String(doc)).toBe(src)
 })
 
@@ -15,14 +15,13 @@ test('re-defined anchor', () => {
   const src = '- &a 1\n- &a 2\n- *a\n'
   const doc = YAML.parseDocument(src)
   expect(doc.errors).toHaveLength(0)
-  const { items } = doc.contents
-  expect(items).toMatchObject([
-    { value: 1 },
-    { value: 2 },
-    { source: { value: 2 } }
+  expect(doc.contents.items).toMatchObject([
+    { anchor: 'a', value: 1 },
+    { anchor: 'a', value: 2 },
+    { source: 'a' }
   ])
-  expect(items[2].source).toBe(items[1])
-  expect(String(doc)).toBe('- &a1 1\n- &a 2\n- *a\n')
+  expect(doc.toJS()).toMatchObject([1, 2, 2])
+  expect(String(doc)).toBe('- &a 1\n- &a 2\n- *a\n')
 })
 
 test('circular reference', () => {
@@ -30,56 +29,134 @@ test('circular reference', () => {
   const doc = YAML.parseDocument(src)
   expect(doc.errors).toHaveLength(0)
   expect(doc.warnings).toHaveLength(0)
-  const { items } = doc.contents
-  expect(items).toHaveLength(2)
-  expect(items[1].source).toBe(doc.contents)
-  const res = doc.toJSON()
+  expect(doc.contents).toMatchObject({
+    anchor: 'a',
+    items: [{ value: 1 }, { source: 'a' }]
+  })
+  const res = doc.toJS()
   expect(res[1]).toBe(res)
   expect(String(doc)).toBe(src)
 })
 
+describe('anchor on tagged collection', () => {
+  test('!!set', () => {
+    const res = YAML.parse('- &a !!set { 1, 2 }\n- *a\n')
+    expect(Array.from(res[0])).toMatchObject([1, 2])
+    expect(res[1]).toBe(res[0])
+  })
+
+  test('!!omap', () => {
+    const res = YAML.parse('- &a !!omap [ 1: 1, 2: 2 ]\n- *a\n')
+    expect(Array.from(res[0])).toMatchObject([
+      [1, 1],
+      [2, 2]
+    ])
+    expect(res[1]).toBe(res[0])
+  })
+})
+
 describe('create', () => {
-  test('doc.anchors.setAnchor', () => {
+  test('node.anchor', () => {
     const doc = YAML.parseDocument('[{ a: A }, { b: B }]')
-    const [a, b] = doc.contents.items
-    expect(doc.anchors.setAnchor(null, null)).toBe(null)
-    expect(doc.anchors.setAnchor(a, 'XX')).toBe('XX')
-    expect(doc.anchors.setAnchor(a, 'AA')).toBe('AA')
-    expect(doc.anchors.setAnchor(a, 'AA')).toBe('AA')
-    expect(doc.anchors.setAnchor(a)).toBe('AA')
-    expect(doc.anchors.setAnchor(a.items[0].value)).toBe('a1')
-    expect(doc.anchors.setAnchor(b.items[0].value)).toBe('a2')
-    expect(doc.anchors.setAnchor(null, 'a1')).toBe('a1')
-    expect(doc.anchors.getName(a)).toBe('AA')
-    expect(doc.anchors.getNode('a2').value).toBe('B')
-    expect(String(doc)).toBe('[ &AA { a: A }, { b: &a2 B } ]\n')
-    expect(() => doc.anchors.setAnchor(a.items[0])).toThrow(
-      'Anchors may only be set for Scalar, Seq and Map nodes'
-    )
-    expect(() => doc.anchors.setAnchor(a, 'A A')).toThrow(
-      'Anchor names must not contain whitespace or control characters'
-    )
-    expect(doc.anchors.getNames()).toMatchObject(['AA', 'a1', 'a2'])
+    doc.get(0).anchor = 'AA'
+    doc.getIn([0, 'a'], true).anchor = 'a'
+    doc.getIn([0, 'a'], true).anchor = ''
+    doc.getIn([1, 'b'], true).anchor = 'BB'
+    expect(String(doc)).toBe('[ &AA { a: A }, { b: &BB B } ]\n')
   })
 
-  test('doc.anchors.createAlias', () => {
+  test('doc.createAlias', () => {
     const doc = YAML.parseDocument('[{ a: A }, { b: B }]')
-    const alias = doc.anchors.createAlias(doc.contents.items[0], 'AA')
+    const alias = doc.createAlias(doc.get(0), 'AA')
     doc.contents.items.push(alias)
-    expect(doc.toJSON()).toMatchObject([{ a: 'A' }, { b: 'B' }, { a: 'A' }])
-    expect(String(doc)).toMatch('[ &AA { a: A }, { b: B }, *AA ]\n')
+    expect(doc.toJS()).toMatchObject([{ a: 'A' }, { b: 'B' }, { a: 'A' }])
+    const alias2 = doc.createAlias(doc.get(1), 'AA')
+    expect(doc.get(1).anchor).toBe('AA1')
+    expect(alias2.source).toBe('AA1')
+    expect(String(doc)).toMatch('[ &AA { a: A }, &AA1 { b: B }, *AA ]\n')
   })
 
-  test('errors', () => {
+  test('repeated Array in createNode', () => {
+    const doc = new YAML.Document()
+    const ref = []
+    const node = doc.createNode({ src: ref, ref })
+    expect(node).toMatchObject({
+      items: [
+        { key: { value: 'src' }, value: { items: [], anchor: 'a1' } },
+        { key: { value: 'ref' }, value: { source: 'a1' } }
+      ]
+    })
+  })
+
+  test('repeated Date in createNode', () => {
+    const doc = new YAML.Document()
+    const date = new Date()
+    const value = date.toJSON()
+    const node = doc.createNode({ src: date, ref: date })
+    expect(node).toMatchObject({
+      items: [
+        { key: { value: 'src' }, value: { value, anchor: 'a1' } },
+        { key: { value: 'ref' }, value: { source: 'a1' } }
+      ]
+    })
+  })
+})
+
+describe('errors', () => {
+  test('invalid anchor characters', () => {
+    const doc = YAML.parseDocument('[{ a: A }, { b: B }]')
+    doc.get(0).anchor = 'A A'
+    expect(() => String(doc)).toThrow(
+      'Anchor must not contain whitespace or control characters: "A A"'
+    )
+  })
+
+  test('set tag on alias', () => {
     const doc = YAML.parseDocument('[{ a: A }, { b: B }]')
     const node = doc.contents.items[0]
-    const alias = doc.anchors.createAlias(node, 'AA')
-    doc.contents.items.unshift(alias)
-    expect(() => String(doc)).toThrow('Alias node must be after source node')
+    const alias = doc.createAlias(node, 'AA')
     expect(() => {
       alias.tag = 'tag:yaml.org,2002:alias'
     }).toThrow('Alias nodes cannot have tags')
   })
+
+  test('alias before anchor', () => {
+    const doc = YAML.parseDocument('[{ a: A }, { b: B }]')
+    const alias = doc.createAlias(doc.get(0), 'AA')
+    doc.contents.items.unshift(alias)
+    expect(() => String(doc)).toThrow(
+      'Unresolved alias (the anchor must be set before the alias): AA'
+    )
+  })
+
+  test('empty alias', () => {
+    const doc = YAML.parseDocument('- *\n')
+    expect(doc.errors).toMatchObject([{ code: 'BAD_ALIAS' }])
+  })
+
+  test('empty anchor on value', () => {
+    const doc = YAML.parseDocument('- & foo\n')
+    expect(doc.errors).toMatchObject([{ code: 'BAD_ALIAS' }])
+  })
+
+  test('empty anchor at end', () => {
+    const doc = YAML.parseDocument('- &\n')
+    expect(doc.errors).toMatchObject([{ code: 'BAD_ALIAS' }])
+  })
+})
+
+describe('warnings', () => {
+  test('anchor ending in colon', () => {
+    const doc = YAML.parseDocument('- &x:\n')
+    expect(doc.errors).toHaveLength(0)
+    expect(doc.warnings).toMatchObject([{ code: 'BAD_ALIAS' }])
+  })
+
+  test('alias ending in colon', () => {
+    const doc = YAML.parseDocument('- *x:\n')
+    expect(doc.errors).toHaveLength(0)
+    expect(doc.warnings).toMatchObject([{ code: 'BAD_ALIAS' }])
+  })
 })
 
 describe('__proto__ as anchor name', () => {
@@ -87,15 +164,17 @@ describe('__proto__ as anchor name', () => {
     const src = `- &__proto__ 1\n- *__proto__\n`
     const doc = YAML.parseDocument(src)
     expect(doc.errors).toHaveLength(0)
-    const { items } = doc.contents
-    expect(items).toMatchObject([{ value: 1 }, { source: { value: 1 } }])
-    expect(items[1].source).toBe(items[0])
+    expect(doc.contents.items).toMatchObject([
+      { anchor: '__proto__', value: 1 },
+      { source: '__proto__' }
+    ])
+    expect(doc.toJS()).toMatchObject([1, 1])
     expect(String(doc)).toBe(src)
   })
 
   test('create/stringify', () => {
     const doc = YAML.parseDocument('[{ a: A }, { b: B }]')
-    const alias = doc.anchors.createAlias(doc.contents.items[0], '__proto__')
+    const alias = doc.createAlias(doc.contents.items[0], '__proto__')
     doc.contents.items.push(alias)
     expect(doc.toJSON()).toMatchObject([{ a: 'A' }, { b: 'B' }, { a: 'A' }])
     expect(String(doc)).toMatch(
@@ -149,52 +228,76 @@ describe('merge <<', () => {
     }
   })
 
-  test('YAML.parseAllDocuments', () => {
+  test('YAML.parseDocument', () => {
     const doc = YAML.parseDocument(src, { merge: true })
-    expect(doc.contents.items).toHaveLength(8)
-    expect(Object.keys(doc.anchors.map)).toMatchObject([
-      'CENTER',
-      'LEFT',
-      'BIG',
-      'SMALL'
+    expect(
+      doc.contents.items.slice(5).map(it => it.items[0].value)
+    ).toMatchObject([
+      { source: 'CENTER' },
+      { items: [{ source: 'CENTER' }, { source: 'BIG' }] },
+      { items: [{ source: 'BIG' }, { source: 'LEFT' }, { source: 'SMALL' }] }
     ])
-    doc.contents.items.slice(5).forEach(({ items }) => {
-      const merge = items[0]
-      expect(merge).toBeInstanceOf(Merge)
-      merge.value.items.forEach(({ source }) => {
-        expect(source).toBeInstanceOf(YAMLMap)
+  })
+
+  test('alias is associated with a sequence', () => {
+    const res = YAML.parse(
+      '- &items [{ a: A }, { b: B }]\n- { <<: *items, c: C }',
+      {
+        merge: true
+      }
+    )
+    expect(res[1]).toEqual({ a: 'A', b: 'B', c: 'C' })
+  })
+
+  describe('non-alias merges', () => {
+    test('single map', () => {
+      const res = YAML.parse('{ <<: { a: A }, b: B }', { merge: true })
+      expect(res).toEqual({ a: 'A', b: 'B' })
+    })
+
+    test('multiple maps', () => {
+      const res = YAML.parse('{ <<: [{ a: A }, { b: B }], c: C }', {
+        merge: true
       })
+      expect(res).toEqual({ a: 'A', b: 'B', c: 'C' })
+    })
+
+    test('mixed refs & maps', () => {
+      const res = YAML.parse('{ <<: [{ a: &a { A: 1 } }, *a], b: B }', {
+        merge: true
+      })
+      expect(res).toEqual({ a: { A: 1 }, A: 1, b: 'B' })
     })
   })
 
-  describe('doc.anchors.createMergePair', () => {
+  describe('create', () => {
     test('simple case', () => {
-      const doc = YAML.parseDocument('[{ a: A }, { b: B }]')
+      const doc = YAML.parseDocument('[{ a: A }, { b: B }]', { merge: true })
       const [a, b] = doc.contents.items
-      const merge = doc.anchors.createMergePair(a)
+      const merge = doc.createPair('<<', doc.createAlias(a))
       b.items.push(merge)
-      expect(doc.toJSON()).toMatchObject([{ a: 'A' }, { a: 'A', b: 'B' }])
+      expect(doc.toJS()).toMatchObject([{ a: 'A' }, { a: 'A', b: 'B' }])
       expect(String(doc)).toBe('[ &a1 { a: A }, { b: B, <<: *a1 } ]\n')
     })
 
     test('merge pair of an alias', () => {
-      const doc = YAML.parseDocument('[{ a: A }, { b: B }]')
+      const doc = YAML.parseDocument('[{ a: A }, { b: B }]', { merge: true })
       const [a, b] = doc.contents.items
-      const alias = doc.anchors.createAlias(a, 'AA')
-      const merge = doc.anchors.createMergePair(alias)
+      const alias = doc.createAlias(a, 'AA')
+      const merge = doc.createPair('<<', alias)
       b.items.push(merge)
-      expect(doc.toJSON()).toMatchObject([{ a: 'A' }, { a: 'A', b: 'B' }])
+      expect(doc.toJS()).toMatchObject([{ a: 'A' }, { a: 'A', b: 'B' }])
       expect(String(doc)).toBe('[ &AA { a: A }, { b: B, <<: *AA } ]\n')
     })
 
     test('require map node', () => {
-      const exp = 'Merge sources must be Map nodes or their Aliases'
-      const doc = YAML.parseDocument('[{ a: A }, { b: B }]')
-      const [a] = doc.contents.items
-      const merge = doc.anchors.createMergePair(a)
-      expect(() => doc.anchors.createMergePair(merge)).toThrow(exp)
-      const alias = doc.anchors.createAlias(a.items[0].value)
-      expect(() => doc.anchors.createMergePair(alias)).toThrow(exp)
+      const doc = YAML.parseDocument('[{ a: A }, { b: B }]', { merge: true })
+      const alias = doc.createAlias(doc.getIn([0, 'a'], true))
+      doc.addIn([1], doc.createPair('<<', alias))
+      expect(String(doc)).toBe('[ { a: &a1 A }, { b: B, <<: *a1 } ]\n')
+      expect(() => doc.toJS()).toThrow(
+        'Merge sources must be maps or map aliases'
+      )
     })
   })
 
@@ -286,14 +389,21 @@ y:
     test('non-alias merge value', () => {
       const src = '{ <<: A, B: b }'
       expect(() => YAML.parse(src, { merge: true })).toThrow(
-        'Merge nodes can only have Alias nodes as values'
+        'Merge sources must be maps or map aliases'
       )
     })
 
     test('non-map alias', () => {
       const src = '- &A a\n- { <<: *A, B: b }'
       expect(() => YAML.parse(src, { merge: true })).toThrow(
-        'Merge nodes aliases can only point to maps'
+        'Merge sources must be maps or map aliases'
+      )
+    })
+
+    test('immediate non-map merge value', () => {
+      const src = '{ <<: [ [42] ], a: A }'
+      expect(() => YAML.parse(src, { merge: true })).toThrow(
+        'Merge sources must be maps or map aliases'
       )
     })
 
@@ -302,9 +412,15 @@ y:
       const doc = YAML.parseDocument(src, { merge: true })
       expect(doc.errors).toHaveLength(0)
       expect(doc.warnings).toHaveLength(0)
-      expect(() => doc.toJSON()).toThrow('Maximum call stack size exceeded')
+      expect(() => doc.toJS()).toThrow('Maximum call stack size exceeded')
       expect(String(doc)).toBe(src)
     })
+
+    test('missing whitespace', () => {
+      expect(() => YAML.parse('&a{}')).toThrow(
+        'Tags and anchors must be separated from the next token by white space'
+      )
+    })
   })
 
   describe('stringify', () => {
