@@ -1,6 +1,5 @@
-import { source } from 'common-tags'
-import YAML from '../../index.js'
-import { Pair } from '../../types.js'
+import { source } from '../_utils'
+import * as YAML from 'yaml'
 
 describe('parse comments', () => {
   describe('body', () => {
@@ -20,7 +19,7 @@ describe('parse comments', () => {
       `
       const doc = YAML.parseDocument(src)
       expect(doc.contents.commentBefore).toBe('comment\ncomment')
-      expect(String(doc)).toBe(src + '\n')
+      expect(String(doc)).toBe(src)
     })
 
     test('body start comments with empty comment line', () => {
@@ -32,8 +31,14 @@ describe('parse comments', () => {
         string
       `
       const doc = YAML.parseDocument(src)
-      expect(doc.contents.commentBefore).toBe('comment\n\ncomment')
-      expect(String(doc)).toBe(src + '\n')
+      expect(doc.contents.commentBefore).toBe('comment\n \ncomment')
+      expect(String(doc)).toBe(source`
+        ---
+        #comment
+        #
+        #comment
+        string
+      `)
     })
 
     test('body end comments', () => {
@@ -52,7 +57,7 @@ describe('parse comments', () => {
       expect(doc.contents.comment).toBe('c1')
       expect(doc.comment).toBe('c2')
       expect(doc.contents.value).toBe('value')
-      expect(doc.contents.range).toMatchObject([4, 13])
+      expect(doc.contents.range).toMatchObject([4, 9, 14])
     })
 
     test('"quoted"', () => {
@@ -62,7 +67,7 @@ describe('parse comments', () => {
       expect(doc.contents.comment).toBe('c1')
       expect(doc.comment).toBe('c2')
       expect(doc.contents.value).toBe('value')
-      expect(doc.contents.range).toMatchObject([4, 15])
+      expect(doc.contents.range).toMatchObject([4, 11, 16])
     })
 
     test('block', () => {
@@ -72,47 +77,50 @@ describe('parse comments', () => {
       expect(doc.contents.comment).toBe('c1')
       expect(doc.comment).toBe('c2')
       expect(doc.contents.value).toBe('value')
-      expect(doc.contents.range).toMatchObject([4, 18])
+      expect(doc.contents.range).toMatchObject([4, 18, 18])
     })
   })
 
   describe('seq entry comments', () => {
     test('plain', () => {
-      const src = `#c0
-- value 1
-#c1
+      const src = source`
+        #c0
+        - value 1
+          #c1
 
-- value 2
+        - value 2
 
-#c2`
+        #c2
+      `
       const doc = YAML.parseDocument(src)
       expect(doc).toMatchObject({
         contents: {
           items: [
-            { commentBefore: 'c0', range: [6, 13] },
-            { commentBefore: 'c1' }
+            { commentBefore: 'c0', value: 'value 1', comment: 'c1' },
+            { value: 'value 2' }
           ],
-          range: [4, 29]
+          range: [4, 31, 31]
         },
         comment: 'c2'
       })
     })
 
     test('multiline', () => {
-      const src = `
-- value 1
-#c0
-#c1
+      const src = source`
+        - value 1
+          #c0
+        #c1
 
-#c2
-- value 2
+        #c2
+        - value 2
 
-#c3
-#c4`
+        #c3
+        #c4
+      `
       const doc = YAML.parseDocument(src)
       expect(doc).toMatchObject({
         contents: {
-          items: [{}, { commentBefore: 'c0\nc1\nc2' }]
+          items: [{ comment: 'c0' }, { commentBefore: 'c1\n\nc2' }]
         },
         comment: 'c3\nc4'
       })
@@ -121,36 +129,46 @@ describe('parse comments', () => {
 
   describe('map entry comments', () => {
     test('plain', () => {
-      const src = `#c0
-key1: value 1
-#c1
+      const src = source`
+        #c0
+        key1: value 1
+          #c1
 
-key2: value 2
+        key2: value 2
 
-#c2`
+        #c2
+      `
       const doc = YAML.parseDocument(src)
       expect(doc).toMatchObject({
         contents: {
-          items: [{ commentBefore: 'c0' }, { commentBefore: 'c1' }]
+          items: [
+            { key: { commentBefore: 'c0' }, value: { comment: 'c1' } },
+            { key: {}, value: {} }
+          ]
         },
         comment: 'c2'
       })
     })
 
     test('multiline', () => {
-      const src = `key1: value 1
-#c0
-#c1
+      const src = source`
+        key1: value 1
+          #c0
+        #c1
 
-#c2
-key2: value 2
+        #c2
+        key2: value 2
 
-#c3
-#c4`
+        #c3
+        #c4
+      `
       const doc = YAML.parseDocument(src)
       expect(doc).toMatchObject({
         contents: {
-          items: [{}, { commentBefore: 'c0\nc1\nc2' }]
+          items: [
+            { value: { comment: 'c0' } },
+            { key: { commentBefore: 'c1\n\nc2' } }
+          ]
         },
         comment: 'c3\nc4'
       })
@@ -159,75 +177,126 @@ key2: value 2
 
   describe('map-in-seq comments', () => {
     test('plain', () => {
-      const src = `#c0
-- #c1
-  k1: v1
-  #c2
-  k2: v2 #c3
-#c4
-  k3: v3
-#c5\n`
+      const src = source`
+        #c0
+        - #c1
+          k1: v1
+          #c2
+          k2: v2 #c3
+        #c4
+          k3: v3
+        #c5
+      `
       const doc = YAML.parseDocument(src)
-      expect(doc.contents.items).toMatchObject([
-        {
-          commentBefore: 'c0\nc1',
+      expect(doc).toMatchObject({
+        contents: {
           items: [
-            {},
-            { commentBefore: 'c2', value: { comment: 'c3' } },
-            { commentBefore: 'c4' }
+            {
+              commentBefore: 'c0\nc1',
+              items: [
+                {},
+                { key: { commentBefore: 'c2' }, value: { comment: 'c3' } },
+                { key: { commentBefore: 'c4' } }
+              ]
+            }
           ]
-        }
-      ])
-      expect(doc.comment).toBe('c5')
-      expect(String(doc)).toBe(`#c0
-#c1
-- k1: v1
-  #c2
-  k2: v2 #c3
-  #c4
-  k3: v3
+        },
+        comment: 'c5'
+      })
+      expect(String(doc)).toBe(source`
+        #c0
+        #c1
+        - k1: v1
+          #c2
+          k2: v2 #c3
+          #c4
+          k3: v3
 
-#c5\n`)
+        #c5
+      `)
     })
   })
 
   describe('seq-in-map comments', () => {
     test('plain', () => {
-      const src = `#c0
-k1: #c1
-  - v1
-#c2
-  - v2
-  #c3
-k2:
-  - v3 #c4
-#c5\n`
+      const src = source`
+        #c0
+        k1: #c1
+          - v1
+        #c2
+          - v2
+          #c3
+        k2:
+          - v3 #c4
+        #c5
+      `
       const doc = YAML.parseDocument(src)
-      expect(doc.contents.items).toMatchObject([
-        {
-          comment: 'c1',
-          key: { commentBefore: 'c0', value: 'k1' },
-          value: {
-            items: [{ value: 'v1' }, { commentBefore: 'c2', value: 'v2' }],
-            comment: 'c3'
-          }
+      expect(doc).toMatchObject({
+        contents: {
+          items: [
+            {
+              key: { commentBefore: 'c0', value: 'k1' },
+              value: {
+                commentBefore: 'c1',
+                items: [{ value: 'v1' }, { commentBefore: 'c2', value: 'v2' }],
+                comment: 'c3'
+              }
+            },
+            {
+              key: { value: 'k2' },
+              value: { items: [{ value: 'v3', comment: 'c4' }] }
+            }
+          ]
         },
-        {
-          key: { value: 'k2' },
-          value: { items: [{ value: 'v3', comment: 'c4' }] }
-        }
-      ])
-      expect(doc.comment).toBe('c5')
-      expect(String(doc)).toBe(`#c0
-k1: #c1
-  - v1
-  #c2
-  - v2
-  #c3
-k2:
-  - v3 #c4
+        comment: 'c5'
+      })
+      expect(String(doc)).toBe(source`
+        #c0
+        k1:
+          #c1
+          - v1
+          #c2
+          - v2
+          #c3
+        k2:
+          - v3 #c4
 
-#c5\n`)
+        #c5
+      `)
+    })
+  })
+
+  describe('flow collection commens', () => {
+    test('line comment after , in seq', () => {
+      const doc = YAML.parseDocument(source`
+        [ a, #c0
+          b #c1
+        ]`)
+      expect(doc.contents.items).toMatchObject([
+        { value: 'a', comment: 'c0' },
+        { value: 'b', comment: 'c1' }
+      ])
+    })
+
+    test('line comment after , in map', () => {
+      const doc = YAML.parseDocument(source`
+        { a, #c0
+          b: c, #c1
+          d #c2
+        }`)
+      expect(doc.contents.items).toMatchObject([
+        { key: { value: 'a', comment: 'c0' } },
+        { key: { value: 'b' }, value: { value: 'c', comment: 'c1' } },
+        { key: { value: 'd', comment: 'c2' } }
+      ])
+    })
+
+    test('multi-line comments', () => {
+      const doc = YAML.parseDocument('{ a,\n#c0\n#c1\nb }')
+      expect(doc.contents.items).toMatchObject([
+        { key: { value: 'a' } },
+        { key: { commentBefore: 'c0\nc1', value: 'b' } }
+      ])
     })
   })
 })
@@ -261,7 +330,7 @@ describe('stringify comments', () => {
       const src = 'string'
       const doc = YAML.parseDocument(src)
       doc.contents.comment = 'comment\nlines'
-      expect(String(doc)).toBe('#comment\n#lines\nstring\n')
+      expect(String(doc)).toBe('string\n#comment\n#lines\n')
     })
 
     test('"quoted"', () => {
@@ -289,14 +358,12 @@ describe('stringify comments', () => {
       const doc = YAML.parseDocument(src)
       expect(doc.commentBefore).toBe('c0')
       doc.commentBefore += '\nc1'
-      expect(String(doc)).toBe(
-        source`
-          #c0
-          #c1
-          ---
-          string
-        ` + '\n'
-      )
+      expect(String(doc)).toBe(source`
+        #c0
+        #c1
+        ---
+        string
+      `)
     })
   })
 
@@ -308,36 +375,34 @@ describe('stringify comments', () => {
       doc.contents.items[0].commentBefore = 'c1'
       doc.contents.items[1].commentBefore = 'c2'
       doc.contents.comment = 'c3'
-      expect(String(doc)).toBe(
-        `#c0
-#c1
-- value 1
-#c2
-- value 2
-#c3
-`
-      )
+      expect(String(doc)).toBe(source`
+        #c0
+        #c1
+        - value 1
+        #c2
+        - value 2
+        #c3
+      `)
     })
 
     test('multiline', () => {
       const src = '- value 1\n- value 2\n'
       const doc = YAML.parseDocument(src)
       doc.contents.items[0].commentBefore = 'c0\nc1'
-      doc.contents.items[1].commentBefore = '\nc2\n\nc3'
+      doc.contents.items[1].commentBefore = ' \nc2\n\nc3'
       doc.contents.comment = 'c4\nc5'
-      expect(String(doc)).toBe(
-        `#c0
-#c1
-- value 1
-#
-#c2
-#
-#c3
-- value 2
-#c4
-#c5
-`
-      )
+      expect(String(doc)).toBe(source`
+        #c0
+        #c1
+        - value 1
+        #
+        #c2
+
+        #c3
+        - value 2
+        #c4
+        #c5
+      `)
     })
 
     test('seq-in-map', () => {
@@ -345,21 +410,37 @@ describe('stringify comments', () => {
       const doc = YAML.parseDocument(src)
       doc.contents.items[0].key.commentBefore = 'c0'
       doc.contents.items[0].key.comment = 'c1'
-      doc.contents.items[0].comment = 'c2'
       const seq = doc.contents.items[0].value
+      seq.commentBefore = 'c2'
       seq.items[0].commentBefore = 'c3'
       seq.items[1].commentBefore = 'c4'
       seq.comment = 'c5'
-      expect(String(doc)).toBe(
-        `#c0
-? map #c1
-: #c2
-  #c3
-  - value 1
-  #c4
-  - value 2
-  #c5\n`
-      )
+      expect(String(doc)).toBe(source`
+        #c0
+        map: #c1
+          #c2
+          #c3
+          - value 1
+          #c4
+          - value 2
+          #c5
+        `)
+    })
+
+    test('custom stringifier', () => {
+      const doc = YAML.parseDocument('- a\n- b\n')
+      doc.contents.commentBefore = 'c0'
+      doc.contents.items[0].commentBefore = 'c1'
+      doc.contents.items[1].commentBefore = 'c2\nc3'
+      const commentString = str => str.replace(/^/gm, '// ')
+      expect(doc.toString({ commentString })).toBe(source`
+        // c0
+        // c1
+        - a
+        // c2
+        // c3
+        - b
+      `)
     })
   })
 
@@ -367,47 +448,100 @@ describe('stringify comments', () => {
     test('plain', () => {
       const src = 'key1: value 1\nkey2: value 2\n'
       const doc = YAML.parseDocument(src)
-      doc.contents.items[0].commentBefore = 'c0'
-      doc.contents.items[1].commentBefore = 'c1'
-      doc.contents.items[1].comment = 'c2'
+      doc.contents.items[0].key.commentBefore = 'c0'
+      doc.contents.items[1].key.commentBefore = 'c1'
+      doc.contents.items[1].key.comment = 'c2'
       doc.contents.items[1].value.spaceBefore = true
       doc.contents.comment = 'c3'
-      expect(String(doc)).toBe(`#c0
-key1: value 1
-#c1
-key2: #c2
+      expect(String(doc)).toBe(source`
+        #c0
+        key1: value 1
+        #c1
+        key2: #c2
 
-  value 2
-#c3\n`)
+          value 2
+        #c3
+      `)
     })
 
     test('multiline', () => {
       const src = 'key1: value 1\nkey2: value 2\n'
       const doc = YAML.parseDocument(src)
-      doc.contents.items[0].commentBefore = 'c0\nc1'
-      doc.contents.items[1].commentBefore = '\nc2\n\nc3'
-      doc.contents.items[1].comment = 'c4\nc5'
+      doc.contents.items[0].key.commentBefore = 'c0\nc1'
+      doc.contents.items[1].key.commentBefore = ' \nc2\n\nc3'
+      doc.contents.items[1].key.comment = 'c4\nc5'
       doc.contents.items[1].value.spaceBefore = true
       doc.contents.items[1].value.commentBefore = 'c6'
       doc.contents.comment = 'c7\nc8'
-      expect(String(doc)).toBe(
-        `#c0
-#c1
-key1: value 1
-#
-#c2
-#
-#c3
-key2:
-  #c4
-  #c5
+      expect(String(doc)).toBe(source`
+        #c0
+        #c1
+        key1: value 1
+        #
+        #c2
 
-  #c6
-  value 2
-#c7
-#c8
-`
-      )
+        #c3
+        key2:
+          #c4
+          #c5
+
+          #c6
+          value 2
+        #c7
+        #c8
+      `)
+    })
+
+    test('indented comment on empty value with spaceBefore', () => {
+      const src = source`
+        key1:
+
+          # comment
+        key2: x
+      `
+      const doc = YAML.parseDocument(src)
+      expect(String(doc)).toBe(src)
+    })
+
+    test('comment after empty value with spaceBefore', () => {
+      const src = source`
+        key1:
+
+        # comment
+        key2: x
+      `
+      const doc = YAML.parseDocument(src)
+      expect(String(doc)).toBe(src)
+    })
+  })
+
+  describe('flow collection comments', () => {
+    test('line comment after , in seq', () => {
+      const doc = YAML.parseDocument(source`
+        [ a, #c0
+          b #c1
+        ]`)
+      expect(String(doc)).toBe(source`
+        [
+          a, #c0
+          b #c1
+        ]
+      `)
+    })
+
+    test('line comment after , in map', () => {
+      const doc = YAML.parseDocument(source`
+        { a, #c0
+          b: c, #c1
+          d #c2
+        }`)
+      expect(String(doc)).toBe(source`
+        {
+          a, #c0
+          b: c, #c1
+          d #c2
+        }
+      `)
     })
   })
 })
@@ -421,7 +555,7 @@ describe('blank lines', () => {
     })
 
     test('content comment', () => {
-      const src = '\n\n#cc\n\nstr\n'
+      const src = '\n\n#cc\n \nstr\n'
       const doc = YAML.parseDocument(src)
       expect(String(doc)).toBe('#cc\n\nstr\n')
     })
@@ -479,9 +613,8 @@ describe('blank lines', () => {
 
   test('before first node in document with directives', () => {
     const doc = YAML.parseDocument('str\n')
-    doc.directivesEndMarker = true
     doc.contents.spaceBefore = true
-    expect(String(doc)).toBe('---\n\nstr\n')
+    expect(doc.toString({ directives: true })).toBe('---\n\nstr\n')
   })
 
   test('between seq items', () => {
@@ -491,9 +624,30 @@ describe('blank lines', () => {
   })
 
   test('between seq items with leading comments', () => {
-    const src = '#A\n- a\n\n#B\n- b\n\n\n#C\n\n- c\n'
+    const src = source`
+      #A
+      - a
+
+      #B
+      - b
+
+
+      #C
+
+      - c
+    `
     const doc = YAML.parseDocument(src)
-    expect(String(doc)).toBe('#A\n- a\n\n#B\n- b\n\n#C\n- c\n')
+    expect(String(doc)).toBe(source`
+      #A
+      - a
+
+      #B
+      - b
+
+      #C
+
+      - c
+    `)
   })
 
   describe('not after block scalar with keep chomping', () => {
@@ -506,8 +660,12 @@ describe('blank lines', () => {
       test(name, () => {
         const doc = YAML.parseDocument(src)
         expect(String(doc)).toBe(src)
-        expect(doc.contents.items[1]).not.toHaveProperty('spaceBefore', true)
-        doc.contents.items[1].spaceBefore = true
+        let it = doc.contents.items[1]
+        if (it.key) it = it.key
+        expect(it).not.toHaveProperty('spaceBefore', true)
+        it.spaceBefore = true
+        expect(String(doc)).toBe(src)
+        it.commentBefore = '\n\n'
         expect(String(doc)).toBe(src)
       })
     }
@@ -520,6 +678,8 @@ describe('blank lines', () => {
         contents: { value: 'a\n\n' }
       })
       expect(String(doc)).toBe(src)
+      doc.comment = '\n\nc'
+      expect(String(doc)).toBe(src)
     })
   })
 
@@ -577,8 +737,8 @@ describe('blank lines', () => {
       const doc = YAML.parseDocument(src)
       expect(doc.contents).toMatchObject({
         items: [
-          { key: { value: 'a' }, value: { value: 1 }, spaceBefore: true },
-          { key: { value: 'b' }, value: { value: 2 }, spaceBefore: true }
+          { key: { value: 'a', spaceBefore: true }, value: { value: 1 } },
+          { key: { value: 'b', spaceBefore: true }, value: { value: 2 } }
         ]
       })
     })
@@ -616,23 +776,259 @@ map:
     key3: value3`
     const doc = YAML.parseDocument(src)
     expect(doc.errors).toHaveLength(0)
-    expect(doc.toJSON()).toMatchObject({
+    expect(doc.toJS()).toMatchObject({
       map: { foo0: { key2: 'value2' }, foo2: { key3: 'value3' } }
     })
   })
+
+  describe('commented empty lines', () => {
+    test('document comments with #', () => {
+      const src = source`
+        # c1
+        #
+        # c2
+
+        value
+
+        # c3
+        #
+        # c4
+      `
+      const doc = YAML.parseDocument(src)
+      expect(doc).toMatchObject({
+        commentBefore: ' c1\n \n c2',
+        comment: ' c3\n \n c4'
+      })
+      expect(doc.toString()).toBe(source`
+        # c1
+        #
+        # c2
+
+        value
+
+        # c3
+        #
+        # c4
+      `)
+    })
+
+    test('document comments without #', () => {
+      const src = source`
+        # c1
+
+        # c2
+
+        value
+
+        # c3
+
+        # c4
+      `
+      const doc = YAML.parseDocument(src)
+      expect(doc).toMatchObject({
+        commentBefore: ' c1\n\n c2',
+        comment: ' c3\n\n c4'
+      })
+      expect(doc.toString()).toBe(src)
+    })
+
+    test('map comments with #', () => {
+      const src = source`
+        key:
+          # c1
+          #
+          # c2
+
+          value
+
+          # c3
+          #
+          # c4
+      `
+      const doc = YAML.parseDocument(src)
+      expect(doc.contents).toMatchObject({
+        items: [
+          { value: { commentBefore: ' c1\n \n c2\n', comment: ' c3\n \n c4' } }
+        ]
+      })
+      expect(doc.toString()).toBe(source`
+        key:
+          # c1
+          #
+          # c2
+
+          value
+          # c3
+          #
+          # c4
+      `)
+    })
+
+    test('map comments without #', () => {
+      const src = source`
+        key:
+          # c1
+
+          # c2
+
+          value
+
+          # c3
+
+          # c4
+      `
+      const doc = YAML.parseDocument(src)
+      expect(doc.contents).toMatchObject({
+        items: [
+          { value: { commentBefore: ' c1\n\n c2\n', comment: ' c3\n\n c4' } }
+        ]
+      })
+      expect(doc.toString()).toBe(source`
+        key:
+          # c1
+
+          # c2
+
+          value
+          # c3
+
+          # c4
+      `)
+    })
+
+    test('seq comments with #', () => {
+      const src = source`
+        # c1
+        #
+        # c2
+        - v1
+          # c3
+          #
+        # c4
+        - v2
+      `
+      const doc = YAML.parseDocument(src)
+      expect(doc.contents).toMatchObject({
+        items: [
+          { commentBefore: ' c1\n \n c2', comment: ' c3\n ' },
+          { commentBefore: ' c4' }
+        ]
+      })
+      expect(doc.toString()).toBe(source`
+        # c1
+        #
+        # c2
+        - v1
+          # c3
+          #
+        # c4
+        - v2
+      `)
+    })
+
+    test('seq comments without #', () => {
+      const src = source`
+        # c1
+
+        # c2
+        - v1
+          # c3
+
+        # c4
+        - v2
+      `
+      const doc = YAML.parseDocument(src)
+      expect(doc.contents).toMatchObject({
+        items: [
+          { commentBefore: ' c1\n\n c2', comment: ' c3' },
+          { commentBefore: ' c4', spaceBefore: true }
+        ]
+      })
+      expect(doc.toString()).toBe(source`
+        # c1
+
+        # c2
+        - v1 # c3
+
+        # c4
+        - v2
+      `)
+    })
+  })
+
+  describe('newlines as comments', () => {
+    test('seq', () => {
+      const doc = YAML.parseDocument('- v1\n- v2\n')
+      const [v1, v2] = doc.contents.items
+      v1.commentBefore = '\n'
+      v1.comment = '\n'
+      v2.commentBefore = '\n'
+      v2.comment = '\n'
+      expect(doc.toString()).toBe(source`
+
+        - v1
+
+
+        - v2
+
+      `)
+    })
+
+    test('map', () => {
+      const doc = YAML.parseDocument('k1: v1\nk2: v2')
+      const [p1, p2] = doc.contents.items
+      p1.key.commentBefore = '\n'
+      p1.value.commentBefore = '\n'
+      p1.value.comment = '\n'
+      p2.key.commentBefore = '\n'
+      p2.value.commentBefore = '\n'
+      p2.value.comment = '\n'
+      expect(doc.toString()).toBe(source`
+
+        k1:
+
+          v1
+
+
+        k2:
+
+          v2
+
+      `)
+    })
+  })
+
+  test('eemeli/yaml#277', () => {
+    const src = source`
+      environment:
+        ### SESSION START ###
+        A: "true"
+        B: "true"
+        ### SESSION END ###
+
+        ### ANOTHER SESSION START ###
+        C: "true"
+        D: "true"
+        ### ANOTHER SESSION END ###
+
+    `
+    const doc = YAML.parseDocument(src)
+    expect(doc.toString()).toBe(src)
+  })
 })
 
 describe('eemeli/yaml#18', () => {
   test('reported', () => {
     const src = `test1:
-  foo: #123
+  foo:
+    #123
     bar: 1\n`
     const doc = YAML.parseDocument(src)
     expect(String(doc)).toBe(src)
   })
 
   test('minimal', () => {
-    const src = `foo: #123\n  bar: baz\n`
+    const src = `foo:\n  #123\n  bar: baz\n`
     const doc = YAML.parseDocument(src)
     expect(String(doc)).toBe(src)
   })
@@ -667,19 +1063,6 @@ entryB:
     expect(String(doc)).toBe(`a: b #c\n\n#d\n`)
   })
 
-  test('comment association by indentation', () => {
-    const src = `
-a:
-  - b #c
-#d\n`
-    const cst = YAML.parseCST(src)
-    const collection = cst[0].contents[1]
-    expect(collection.items).toHaveLength(2)
-    const comment = cst[0].contents[2]
-    expect(comment.type).toBe('COMMENT')
-    expect(comment.comment).toBe('d')
-  })
-
   test('blank line after seq in map', () => {
     const src = `a:
   - aa
@@ -706,31 +1089,43 @@ c: cc\n`
 
 describe('collection end comments', () => {
   test('seq in seq', () => {
-    const src = `#0
-- - a
-  - b
-  #1
+    const src = source`
+      #0
+      - - a
+        - b
+        #1
 
-#2
-- d\n`
+      #2
+      - d
+    `
     const doc = YAML.parseDocument(src)
     expect(doc.contents).toMatchObject({
       items: [
-        { items: [{ value: 'a' }, { value: 'b' }], comment: '1' },
-        { spaceBefore: true, commentBefore: '2', value: 'd' }
+        { items: [{ value: 'a' }, { value: 'b' }], comment: '1\n\n2' },
+        { value: 'd' }
       ]
     })
-    expect(String(doc)).toBe(src)
+    expect(String(doc)).toBe(source`
+      #0
+      - - a
+        - b
+        #1
+
+        #2
+      - d
+    `)
   })
 
   test('map in seq', () => {
-    const src = `#0
-- a: 1
-  b: 2
-  #1
+    const src = source`
+      #0
+      - a: 1
+        b: 2
+        #1
 
-#2
-- d\n`
+      #2
+      - d
+    `
     const doc = YAML.parseDocument(src)
     expect(doc.contents).toMatchObject({
       items: [
@@ -739,50 +1134,66 @@ describe('collection end comments', () => {
             { key: { value: 'a' }, value: { value: 1 } },
             { key: { value: 'b' }, value: { value: 2 } }
           ],
-          comment: '1'
+          comment: '1\n\n2'
         },
-        { spaceBefore: true, commentBefore: '2', value: 'd' }
+        { value: 'd' }
       ]
     })
-    expect(String(doc)).toBe(src)
+    expect(String(doc)).toBe(source`
+      #0
+      - a: 1
+        b: 2
+        #1
+
+        #2
+      - d
+    `)
   })
 
   test('seq in map', () => {
-    const src = `#0
-a:
-  - b
-  - c
-  #1
+    const src = source`
+      #0
+      a:
+        - b
+        - c
+        #1
 
-#2
-d: 1\n`
+      #2
+      d: 1
+    `
     const doc = YAML.parseDocument(src)
     expect(doc.contents).toMatchObject({
       items: [
         {
           key: { value: 'a' },
-          value: { items: [{ value: 'b' }, { value: 'c' }], comment: '1' }
+          value: { items: [{ value: 'b' }, { value: 'c' }], comment: '1\n\n2' }
         },
-        {
-          spaceBefore: true,
-          commentBefore: '2',
-          key: { value: 'd' },
-          value: { value: 1 }
-        }
+        { key: { value: 'd' }, value: { value: 1 } }
       ]
     })
-    expect(String(doc)).toBe(src)
+    expect(String(doc)).toBe(source`
+      #0
+      a:
+        - b
+        - c
+        #1
+
+        #2
+      d: 1
+    `)
   })
 
   test('map in map', () => {
-    const src = `#0
-a:
-  b: 1
-  c: 2
-  #1
+    const src = source`
+      #0
+      a:
+        b: 1
+        c: 2
+        #1
 
-#2
-d: 1\n`
+      #2
+      d: 1
+    `
     const doc = YAML.parseDocument(src)
     expect(doc.contents).toMatchObject({
       items: [
@@ -793,18 +1204,22 @@ d: 1\n`
               { key: { value: 'b' }, value: { value: 1 } },
               { key: { value: 'c' }, value: { value: 2 } }
             ],
-            comment: '1'
+            comment: '1\n\n2'
           }
         },
-        {
-          spaceBefore: true,
-          commentBefore: '2',
-          key: { value: 'd' },
-          value: { value: 1 }
-        }
+        { key: { value: 'd' }, value: { value: 1 } }
       ]
     })
-    expect(String(doc)).toBe(src)
+    expect(String(doc)).toBe(source`
+      #0
+      a:
+        b: 1
+        c: 2
+        #1
+
+        #2
+      d: 1
+    `)
   })
 
   test('indented seq in map in seq', () => {
@@ -841,34 +1256,3 @@ a:
     expect(String(doc)).toBe(src)
   })
 })
-
-describe('Pair.commentBefore', () => {
-  test('Should get key comment', () => {
-    const key = YAML.createNode('foo', true)
-    const pair = new Pair(key, 42)
-    key.commentBefore = 'cc'
-    expect(pair.commentBefore).toBe('cc')
-  })
-
-  test('Should set key comment', () => {
-    const key = YAML.createNode('foo', true)
-    const pair = new Pair(key, 42)
-    pair.commentBefore = 'cc'
-    expect(key.commentBefore).toBe('cc')
-  })
-
-  test('Should create a key from a null value', () => {
-    const pair = new Pair(null, 42)
-    expect(pair.key).toBeNull()
-    pair.commentBefore = 'cc'
-    expect(pair.key).not.toBeNull()
-    expect(pair.key.commentBefore).toBe('cc')
-  })
-
-  test('Should throw for non-Node key', () => {
-    const pair = new Pair({ foo: 'bar' }, 42)
-    expect(() => {
-      pair.commentBefore = 'cc'
-    }).toThrow(/commentBefore is an alias/)
-  })
-})
